@@ -21,14 +21,15 @@ from .config import HardwareInformation
 from .config import OperatingSystem
 from .config import RuntimeEnvironment
 from .enums import ThothAdviserIntegrationEnum
+from .helpers import Lazy
 from .helpers import cwd
 from .helpers import datetime2datetime_str
 from .helpers import datetime_str2timestamp
 from .helpers import datetime_str_from_timestamp
+from .helpers import get_justification_link
 from .helpers import get_service_account_token
 from .helpers import parse_datetime
 from .helpers import timestamp2datetime
-from .helpers import Lazy
 from .json import SafeJSONEncoder
 from .logging import init_logging
 from .openshift import OpenShift
@@ -36,7 +37,7 @@ from .workflows import Workflow
 from .workflows import WorkflowManager
 
 __name__ = "thoth-common"
-__version__ = "0.16.1"
+__version__ = "0.20.1"
 
 
 __all__ = [
@@ -44,17 +45,18 @@ __all__ = [
     "datetime2datetime_str",
     "datetime_str2timestamp",
     "datetime_str_from_timestamp",
+    "get_justification_link",
     "get_service_account_token",
     "HardwareInformation",
     "init_logging",
+    "Lazy",
     "OpenShift",
     "OperatingSystem",
     "parse_datetime",
     "RuntimeEnvironment",
     "SafeJSONEncoder",
     "ThothAdviserIntegrationEnum",
+    "timestamp2datetime",
     "Workflow",
     "WorkflowManager",
-    "timestamp2datetime",
-    "Lazy",
 ]
