@@ -33,6 +33,7 @@ from typing import List
 from typing import Optional
 
 from openshift.dynamic.exceptions import NotFoundError as OpenShiftNotFoundError
+from .exceptions import ThothCommonException
 from .exceptions import NotKnownThothIntegration
 from .exceptions import QebHwtInputsMissing
 from .exceptions import KebechetInputsMissing
@@ -123,6 +124,11 @@ class OpenShift:
 
         self.configuration = self.ocp_client.configuration
 
+        # TODO: Update openshift.
+        # These parameters are missing in openshift configuration, but required for Argo API validation
+        self.configuration.retries = 5
+        self.configuration.client_side_validation = True
+
         self.amun_inspection_namespace = amun_inspection_namespace or os.getenv(
             "THOTH_AMUN_INSPECTION_NAMESPACE"
         )
@@ -307,7 +313,10 @@ class OpenShift:
                 )
 
     def get_pod_log(
-        self, pod_id: str, namespace: Optional[str] = None
+        self,
+        pod_id: str,
+        namespace: Optional[str] = None,
+        container: Optional[str] = None,
     ) -> Optional[str]:
         """Get log of a pod based on assigned pod ID."""
         if not namespace:
@@ -317,9 +326,13 @@ class OpenShift:
                 )
             namespace = self.middletier_namespace
 
+        params = None
+        if container:
+            params = {"container": container}
+
         # TODO: rewrite to OpenShift rest client once it will support it.
         endpoint = "{}/api/v1/namespaces/{}/pods/{}/log".format(
-            self.openshift_api_url, namespace, pod_id
+            self.openshift_api_url, namespace, pod_id,
         )
 
         response = requests.get(
@@ -329,6 +342,7 @@ class OpenShift:
                 "Content-Type": "application/json",
             },
             verify=self.kubernetes_verify_tls,
+            params=params,
         )
         _LOGGER.debug(
             "Kubernetes master response for pod log (%d): %r",
@@ -341,13 +355,43 @@ class OpenShift:
                 f"Pod with id {pod_id} was not found in namespace {namespace}"
             )
 
-        if response.status_code == 400:
+        if (
+            response.status_code == 400
+            and "a container name must be specified for pod"
+            not in response.json()["message"]
+        ):
             # If Pod has not been initialized yet, there is returned 400 status code. Return None in this case.
             return None
+        elif response.status_code == 400:
+            raise ThothCommonException(
+                f"Failed to obtain logs, container name has to be specified: {response.json()['message']}"
+            )
+
+        try:
+            response.raise_for_status()
+        except Exception as exc:
+            _LOGGER.error("Error response when obtaining pod logs: %r", response.json())
+            raise ThothCommonException(
+                f"Failed to obtain logs for pod: {str(exc)}"
+            ) from exc
 
-        response.raise_for_status()
         return response.text
 
+    def get_workflow_node_log(
+        self, node_name: str, workflow_id: str, namespace: str
+    ) -> Optional[str]:
+        """Get log from a task/node in a workflow."""
+        workflow = self.get_workflow(workflow_id, namespace=namespace)
+        nodes = workflow.get("status", {}).get("nodes", {})
+
+        for pod_name, node_info in nodes.items():
+            if node_info["displayName"] == node_name:
+                return self.get_pod_log(pod_name, namespace=namespace, container="main")
+
+        raise NotFoundException(
+            f"No node {node_name!r} in workflow {workflow_id!r} in namespace {namespace!r} found"
+        )
+
     def get_build(self, build_id: str, namespace: str) -> Dict[str, Any]:
         """Get a build in the given namespace."""
         # TODO: rewrite to OpenShift rest client once it will support it.
@@ -680,7 +724,9 @@ class OpenShift:
             _label_selector,
             response.to_dict(),
         )
-        self._raise_on_invalid_response_size(response)
+        self._raise_on_invalid_response_size(
+            response, label_selector=_label_selector, namespace=namespace
+        )
         template: Dict[str, Any] = response.to_dict()["items"][0]
         return template
 
@@ -694,7 +740,9 @@ class OpenShift:
             _label_selector,
             response.to_dict(),
         )
-        self._raise_on_invalid_response_size(response)
+        self._raise_on_invalid_response_size(
+            response, label_selector=_label_selector, namespace=namespace
+        )
         template: Dict[str, Any] = response.to_dict()["items"][0]
         return template
 
@@ -739,6 +787,7 @@ class OpenShift:
         parameters: Dict[str, Any],
         *,
         raw_specification: Optional[Dict[str, Any]] = None,
+        job_id: Optional[str] = None,
     ) -> Optional[str]:
         """Schedule an inspection run."""
         if not self.amun_inspection_namespace:
@@ -746,7 +795,7 @@ class OpenShift:
                 "Unable to schedule inspection without Amun inspection namespace being set."
             )
 
-        inspection_id = self.generate_id(
+        inspection_id = job_id or self.generate_id(
             prefix="inspection", identifier=specification.get("identifier")
         )
 
@@ -768,9 +817,9 @@ class OpenShift:
         workflow_parameters["dockerfile"] = dockerfile
         # Propagate raw specification to be placed on Ceph, not to submit escaped specification.
         workflow_parameters["specification"] = (
-            json.dumps(raw_specification)
+            json.dumps(raw_specification, sort_keys=True, indent=2)
             if raw_specification is not None
-            else json.dumps(specification)
+            else json.dumps(specification, sort_keys=True, indent=2)
         )
         workflow_parameters["target"] = target
 
@@ -999,6 +1048,8 @@ class OpenShift:
         context: Dict[str, Any],
         *,
         pipeline: Optional[Dict[str, Any]] = None,
+        predictor: Optional[str] = None,
+        predictor_config: Optional[Dict[str, Any]] = None,
         stack_output: Optional[str] = None,
         runtime_environment: Optional[Dict[Any, Any]] = None,
         seed: Optional[int] = None,
@@ -1028,6 +1079,10 @@ class OpenShift:
             "THOTH_DEPENDENCY_MONKEY_JOB_ID": job_id,
             "THOTH_DOCUMENT_ID": job_id,
             "THOTH_ADVISER_PIPELINE": json.dumps(pipeline) if pipeline else "{}",
+            "THOTH_ADVISER_PREDICTOR": predictor or "AUTO",
+            "THOTH_ADVISER_PREDICTOR_CONFIG": json.dumps(predictor_config)
+            if predictor_config
+            else "{}",
         }
 
         if decision is not None:
@@ -1161,6 +1216,7 @@ class OpenShift:
         *,
         count: Optional[int] = None,
         limit: Optional[int] = None,
+        predictor_config: Optional[Dict[str, Any]] = None,
         runtime_environment: Optional[Dict[Any, Any]] = None,
         library_usage: Optional[Dict[Any, Any]] = None,
         origin: Optional[str] = None,
@@ -1202,7 +1258,6 @@ class OpenShift:
             template_parameters[
                 "THOTH_ADVISER_REQUIREMENTS_LOCKED"
             ] = application_stack["requirements_lock"]
-        template_parameters["IMAGE_STREAM_NAMESPACE"] = self.infra_namespace
         template_parameters["THOTH_ADVISER_JOB_ID"] = adviser_id
         template_parameters["THOTH_ADVISER_DEV"] = "1" if dev else "0"
         template_parameters["THOTH_ADVISER_REQUIREMENTS"] = application_stack[
@@ -1217,6 +1272,9 @@ class OpenShift:
         template_parameters["THOTH_ADVISER_RUNTIME_ENVIRONMENT"] = json.dumps(
             runtime_environment
         )
+        template_parameters["THOTH_ADVISER_PREDICTOR_CONFIG"] = (
+            json.dumps(predictor_config) if predictor_config else "{}"
+        )
 
         template_parameters["THOTH_ADVISER_METADATA"] = json.dumps(
             {
@@ -1299,7 +1357,7 @@ class OpenShift:
             },
         )
 
-    def schedule_thamos_workflow(
+    def schedule_qebhwt_workflow(
         self,
         github_event_type: str,
         github_check_run_id: int,
@@ -1309,9 +1367,11 @@ class OpenShift:
         origin: str,
         revision: str,
         host: str,
+        *,
+        job_id: Optional[str] = None,
     ) -> Optional[str]:
-        """Schedule Thamos Advise Workflow for Qeb-Hwt GitHub App.."""
-        workflow_id = self.generate_id("qeb-hwt")
+        """Schedule Workflow for Qeb-Hwt GitHub App.."""
+        workflow_id = job_id or self.generate_id("qeb-hwt")
         template_parameters = {
             "WORKFLOW_ID": workflow_id,
             "GITHUB_EVENT_TYPE": github_event_type,
@@ -1324,20 +1384,24 @@ class OpenShift:
             "THOTH_HOST": host,
         }
 
+        workflow_parameters = self._assign_workflow_parameters_for_ceph()
+
         return self._schedule_workflow(
-            workflow=self.workflow_manager.submit_thamos,
+            workflow=self.workflow_manager.submit_qebhwt,
             parameters={
                 "template_parameters": template_parameters,
-                "workflow_parameters": {},
+                "workflow_parameters": workflow_parameters,
             },
         )
 
-    def schedule_mi_workflow(self, repository: str) -> Optional[str]:
+    def schedule_mi_workflow(
+        self, repository: str, *, job_id: Optional[str] = None,
+    ) -> Optional[str]:
         """Schedule Meta-information Indicators Workflow.
 
         :param repository:str: GitHub repository in full name format: <repo_owner>/<repo_name>
         """
-        workflow_id = self.generate_id("mi")
+        workflow_id = job_id or self.generate_id("mi")
         template_parameters = {"WORKFLOW_ID": workflow_id, "REPOSITORY": repository}
 
         return self._schedule_workflow(
@@ -1349,10 +1413,10 @@ class OpenShift:
         )
 
     def schedule_kebechet_workflow(
-        self, webhook_payload: Dict[str, Any]
+        self, webhook_payload: Dict[str, Any], *, job_id: Optional[str] = None,
     ) -> Optional[str]:
         """Schedule Kebechet Workflow for a Webhook from GitHub App.."""
-        workflow_id = self.generate_id("kebechet-job")
+        workflow_id = job_id or self.generate_id("kebechet-job")
         template_parameters = {
             "WORKFLOW_ID": workflow_id,
             "WEBHOOK_PAYLOAD": json.dumps(webhook_payload),
@@ -1399,12 +1463,20 @@ class OpenShift:
             },
         )
 
-    def _raise_on_invalid_response_size(self, response: Any) -> None:
+    def _raise_on_invalid_response_size(
+        self,
+        response: Any,
+        *,
+        label_selector: Optional[str] = None,
+        name: Optional[str] = None,
+        namespace: Optional[str] = None,
+    ) -> None:
         """Expect that there is only one object type for the given item."""
         if len(response.items) != 1:
             raise RuntimeError(
-                f"Application misconfiguration - number of templates available in the infra namespace "
-                f"{self.infra_namespace!r} is {len(response.items)}, should be 1."
+                f"Application misconfiguration - number of templates available in namespace {namespace!r} "
+                f"is {len(response.items)}, should be 1: "
+                f"label selector: {label_selector!r}, name: {name!r}"
             )
 
             # TODO add name of template we were looking for...
@@ -1604,7 +1676,9 @@ class OpenShift:
                     f"The given Workflow containing label {label_selector} could not be found"
                 ) from exc
 
-            self._raise_on_invalid_response_size(response)
+            self._raise_on_invalid_response_size(
+                response, label_selector=label_selector, namespace=namespace
+            )
 
             wf = response.to_dict()["items"][0]
 
